import pytest

from jelastic_client import (
    ControlClient,
    NodeSettings,
    EnvSettings,
    DockerSettings
)
from jelastic_client.core import JelasticClientException


def test_control_client_delete_non_existent_environment_raises_exception(
        control_client: ControlClient,
        non_existent_env_name: str):
    # Arrange

    # Act / Assert
    with pytest.raises(JelasticClientException):
        control_client.delete_env(non_existent_env_name)


def test_control_client_create_environment_with_single_vps_node_runs_environment(
        control_client: ControlClient,
        new_env_name: str):
    # Arrange
    env = EnvSettings(shortdomain=new_env_name)
    node = NodeSettings(flexibleCloudlets=4, nodeType="ubuntu-vps")

    # Act
    created_env_info = control_client.create_environment(env, [node])

    # Assert
    actual_env_name = created_env_info.env_name()
    assert new_env_name == actual_env_name
    actual_env_info = control_client.get_env_info(actual_env_name)
    assert actual_env_info.is_running()


def test_control_client_create_environment_with_single_sql_node_runs_environment(
        control_client: ControlClient,
        new_env_name: str):
    # Arrange
    env = EnvSettings(shortdomain=new_env_name)
    # postgres needs at least 3 cloudlets
    node = NodeSettings(fixedCloudlets=3, flexibleCloudlets=4,
                        nodeType="postgresql")

    # Act
    created_env_info = control_client.create_environment(env, [node])

    # Assert
    actual_env_name = created_env_info.env_name()
    assert new_env_name == actual_env_name
    actual_env_info = control_client.get_env_info(actual_env_name)
    assert actual_env_info.is_running()


def test_control_client_create_environment_with_single_docker_node_runs_environment(
        control_client: ControlClient,
        new_env_name: str):
    # Arrange
    env = EnvSettings(shortdomain=new_env_name)
    docker_settings = DockerSettings(image="alpine")
    node = NodeSettings(docker=docker_settings,
                        flexibleCloudlets=4, nodeType="docker")

    # Act
    created_env_info = control_client.create_environment(env, [node])

    # Assert
    actual_env_name = created_env_info.env_name()
    assert new_env_name == actual_env_name
    actual_env_info = control_client.get_env_info(actual_env_name)
    assert actual_env_info.is_running()


def test_control_client_create_environment_with_multiple_nodes_runs_environment(
        control_client: ControlClient,
        new_env_name: str):
    # Arrange
    env = EnvSettings(shortdomain=new_env_name)
    # postgres needs at least 3 cloudlets
    sql_node = NodeSettings(fixedCloudlets=3, flexibleCloudlets=4,
                            nodeType="postgresql")
    docker_settings = DockerSettings(image="alpine")
    docker_node = NodeSettings(docker=docker_settings,
                               flexibleCloudlets=4, nodeType="docker")

    # Act
    created_env_info = control_client.create_environment(
        env, [sql_node, docker_node])

    # Assert
    actual_env_name = created_env_info.env_name()
    assert new_env_name == actual_env_name
    actual_env_info = control_client.get_env_info(actual_env_name)
    assert actual_env_info.is_running()



import pytest

from jelastic_client import ControlClient, JpsClient
from jelastic_client.core.exceptions import JelasticClientException
from test_utils.manifest_data import get_manifest_data


def test_jps_client_install_from_file_valid_manifest_creates_environment(
        control_client: ControlClient,
        jps_client: JpsClient,
        valid_manifest: str,
        new_env_name: str):
    # Arrange

    # Act
    jps_client.install_from_file(valid_manifest, new_env_name)

    # Assert
    env_info = control_client.get_env_info(new_env_name)
    assert env_info.exists()


def test_jps_client_install_from_file_valid_manifest_makes_environment_run(
        control_client: ControlClient,
        jps_client: JpsClient,
        valid_manifest: str,
        new_env_name: str):
    # Arrange

    # Act
    jps_client.install_from_file(valid_manifest, new_env_name)

    # Assert
    env_info = control_client.get_env_info(new_env_name)
    assert env_info.is_running()


def test_jps_client_install_from_file_valid_manifest_returns_success_text(
        jps_client: JpsClient,
        valid_manifest: str,
        new_env_name: str):
    # Arrange

    # Act
    actual_success_text = jps_client.install_from_file(
        valid_manifest, new_env_name)

    # Assert
    expected_success_text = "<strong>Field1</strong>: Value1<br />\n<strong>Field2</strong>: Value2"
    assert expected_success_text == actual_success_text


def test_jps_client_install_from_file_invalid_manifest_raises_exception(
        jps_client: JpsClient,
        invalid_manifest: str,
        new_env_name: str):
    # Arrange

    # Act / Assert
    with pytest.raises(JelasticClientException):
        jps_client.install_from_file(invalid_manifest, new_env_name)


def test_jps_client_install_from_file_non_existent_manifest_raises_exception(
        jps_client: JpsClient,
        non_existent_manifest: str,
        new_env_name: str):
    # Arrange

    # Act / Assert
    with pytest.raises(OSError):
        jps_client.install_from_file(non_existent_manifest, new_env_name)


def test_jps_client_install_from_file_manifest_with_settings_takes_settings_into_account(
        jps_client: JpsClient,
        manifest_with_settings: str):
    # Arrange
    expected_settings = {
        "field1": "the value 1",
        "field2": "the value 2"
    }

    # Act
    success_text = jps_client.install_from_file(
        manifest_with_settings,
        settings=expected_settings)
    manifest_data = get_manifest_data(success_text)

    # Assert
    for field in expected_settings:
        assert field in manifest_data
        assert expected_settings[field] == manifest_data[field]


def test_jps_client_get_engine_version_returns_supported_engine_version(
        jps_client: JpsClient,
        supported_jelastic_version: str):
    # Arrange

    # Act
    actual_version = jps_client.get_engine_version()

    # Assert
    assert supported_jelastic_version == actual_version



import json

from .core import ApiClient, BaseClient, who_am_i


class JpsClient(BaseClient):
    jelastic_group = "marketplace"

    jelastic_class = "jps"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def install_from_file(self, filename: str, env_name: str = None, settings: dict = None) -> str:
        with open(filename) as file:
            manifest_content = file.read()
            return self.install(manifest_content, env_name, settings)

    def install(self, manifest_content: str, env_name: str = None, settings: dict = None) -> str:
        response = self._execute(
            who_am_i(),
            jps=manifest_content,
            envName=env_name,
            skipNodeEmails=True,
            settings=json.dumps(settings)
        )

        return response["successText"]

    def get_engine_version(self) -> str:
        response = self._execute(
            who_am_i()
        )

        return response["version"]



from typing import NamedTuple


class DockerSettings(NamedTuple):
    image: str
    nodeGroup: str = None


class NodeSettings(NamedTuple):
    docker: DockerSettings = None
    count: int = 1
    displayName: str = None
    extip: bool = False
    fixedCloudlets: int = None
    flexibleCloudlets: int = None
    nodeType: str = None


MultipleNodeSettings = list[NodeSettings]



from enum import Enum


class EnvStatus(Enum):
    Running = 1
    Down = 2
    Launching = 3
    Sleep = 4
    Unknown = 5
    Creating = 6
    Cloning = 7
    NotExists = 8
    Exporting = 9
    Migrating = 10
    Broken = 11
    Updating = 12
    Stopping = 13
    GoingToSleep = 14
    Restoring = 15



from jelastic_client.core import BaseClient, ApiClient, who_am_i


class FileClient(BaseClient):
    jelastic_group = "environment"

    jelastic_class = "file"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def read(self, env_name: str, path: str, node_type: str = None, node_group: str = None, node_id: str = None) -> str:
        response = self._execute(
            who_am_i(),
            envName=env_name,
            path=path,
            nodeType=node_type,
            nodeGroup=node_group,
            nodeid=node_id
        )
        return response["body"]



from abc import ABC, abstractmethod

from .api_client import ApiClient
from .exceptions import JelasticClientException


def success_response(response: dict) -> bool:
    return response["result"] == 0


class BaseClient(ABC):

    def __init__(self, api_client: ApiClient):
        self._api_client = api_client

    @property
    @abstractmethod
    def jelastic_group(self):
        raise NotImplementedError

    @property
    @abstractmethod
    def jelastic_class(self):
        raise NotImplementedError

    def _execute(self, fnc: str, **kwargs) -> dict:
        two_dotted_function_name = self._fnc(fnc)
        response = self._api_client.execute(
            two_dotted_function_name,
            **kwargs
        )

        if not success_response(response):
            raise JelasticClientException(
                f"execution of function {two_dotted_function_name} failed", response)

        return response

    def _fnc(self, fnc_name: str):
        return f"{self.jelastic_group}.{self.jelastic_class}.{fnc_name}"



import pytest

from jelastic_client.env_info import EnvInfo
from jelastic_client.env_status import EnvStatus
from jelastic_client.node import Nodes, Node


@pytest.mark.parametrize(
    "input_dict,expected_status",
    [
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 1
                }
            }, EnvStatus.Running
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 2
                }
            }, EnvStatus.Down
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 5
                }
            }, EnvStatus.Unknown
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 8
                }
            }, EnvStatus.NotExists
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 13
                }
            }, EnvStatus.Stopping
        ),
    ]
)
def test_env_info_status(input_dict: str, expected_status: EnvStatus):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_status = env_info.status()

    # Assert
    assert expected_status == actual_status


def test_env_info_env_name():
    # Arrange
    expected_env_name = "the-env-name"
    input_dict = {
        "nodes": [],
        "env": {
            "envName": expected_env_name
        }
    }
    env_info = EnvInfo(input_dict)

    # Act
    actual_env_name = env_info.env_name()

    # Assert
    assert expected_env_name == actual_env_name


def test_env_info_domain():
    # Arrange
    expected_domain = "local-sha-master-0954606.hidora.com"
    input_dict = {
        "nodes": [],
        "env": {
            "domain": expected_domain
        }
    }
    env_info = EnvInfo(input_dict)

    # Act
    actual_domain = env_info.domain()

    # Assert
    assert expected_domain == actual_domain


@pytest.mark.parametrize(
    "input_dict,expected_running",
    [
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 1
                }
            }, True
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 2
                }
            }, False
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 10
                }
            }, False
        ),
    ]
)
def test_env_info_is_running(input_dict: dict, expected_running: bool):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_running = env_info.is_running()

    # Assert
    assert expected_running == actual_running


@pytest.mark.parametrize(
    "input_dict,expected_exists",
    [
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 1
                }
            }, True
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 2
                }
            }, True
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 10
                }
            }, True
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 8
                }
            }, False
        ),
        pytest.param(
            {
                "nodes": [],
                "env": {
                    "status": 5
                }
            }, False
        ),
    ]
)
def test_env_info_exists(input_dict: dict, expected_exists: bool):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_exists = env_info.exists()

    # Assert
    assert expected_exists == actual_exists


@pytest.mark.parametrize(
    "input_dict,expected_nodes",
    [
        pytest.param(
            {
                "nodes": [],
            }, [], id="empty nodes"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    }
                ],
            }, [Node(int_ip="10.102.15.248", node_type="docker", node_group="cp", url="http://irrelevant.com")],
            id="single node"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "sqldb",
                        "nodeType": "postgresql",
                        "intIP": "10.102.15.249",
                        "url": "http://irrelevant.com"
                    }
                ],
            }, [Node(int_ip="10.102.15.248", node_type="docker", node_group="cp", url="http://irrelevant.com"),
                Node(int_ip="10.102.15.249", node_type="postgresql", node_group="sqldb", url="http://irrelevant.com")],
            id="multiple nodes"
        ),
    ]
)
def test_env_info_nodes(input_dict: dict, expected_nodes: Nodes):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_nodes = env_info.nodes()

    # Assert
    assert not set(actual_nodes) ^ set(expected_nodes)


@pytest.mark.parametrize(
    "input_dict,node_group,node_type,expected_ips",
    [
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "cp", None, ["10.102.15.248"], id="filter list of 1 node on existing node group"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, None, "docker", ["10.102.15.248"], id="filter list of 1 node on existing node type"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, None, "vps", [], id="filter list of 1 node on non-existent node type"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "bl", None, [], id="filter list of 1 node on non-existent node group"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "bl",
                        "nodeType": "docker",
                        "intIP": "10.102.15.258",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.249",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.250",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "cp", None, ["10.102.15.248", "10.102.15.249", "10.102.15.250"],
            id="filter list of multiple nodes on existing node group"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.249",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "ubuntu-vps",
                        "intIP": "10.102.15.259",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.250",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, None, "docker", ["10.102.15.248", "10.102.15.249", "10.102.15.250"],
            id="filter list of multiple nodes on existing node type"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.249",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "ubuntu-vps",
                        "intIP": "10.102.15.259",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "ubuntu-vps",
                        "intIP": "10.102.15.260",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.250",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "cp", "ubuntu-vps", ["10.102.15.259", "10.102.15.260"],
            id="filter list of multiple nodes on existing node type and node group"
        ),
    ],
)
def test_env_info_get_node_ips_returns_filtered_ips(input_dict: dict, node_group: str, node_type: str,
                                                    expected_ips: [str]):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_ips = env_info.get_node_ips(
        node_group=node_group, node_type=node_type)

    # Assert
    assert not set(actual_ips) ^ set(expected_ips)


@pytest.mark.parametrize(
    "input_dict,display_name,expected_ip",
    [
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "target-node",
                        "url": "http://irrelevant.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", "10.102.15.248", id="list of 1 node with desired display name"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.249",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "target-node",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.250",
                        "displayName": "uninteresting-node",
                        "url": "http://irrelevant.com"
                    },
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", "10.102.15.248", id="list of multiple nodes containing the desired display name"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.249",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "other-name",
                        "url": "http://irrelevant.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.250",
                        "displayName": "uninteresting-node",
                        "url": "http://irrelevant.com"
                    },
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", None, id="list of multiple nodes not containing the desired display name"
        ),
    ]
)
def test_env_info_get_node_ip_from_name_returns_corresponding_ip(input_dict: dict, display_name: str, expected_ip: str):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_ip = env_info.get_node_ip_from_name(display_name)

    # Assert
    assert actual_ip == expected_ip


@pytest.mark.parametrize(
    "input_dict,display_name,expected_hostname",
    [
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "target-node",
                        "url": "http://node89876-local-sha-master-0954606.hidora.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", "node89876-local-sha-master-0954606.hidora.com",
            id="list of 1 node with desired display name"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "irrelevant-node",
                        "url": "http://node89876-local-sha-master-0954606.hidora.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", None,
            id="list of 1 node without the desired display name"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "irrelevant-node",
                        "url": "http://node89876-local-sha-master-0954606.hidora.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.268",
                        "displayName": "target-node",
                        "url": "http://node89886-local-sha-master-0954656.hidora.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", "node89886-local-sha-master-0954656.hidora.com",
            id="list of multiple nodes with the desired display name"
        ),
    ]
)
def test_env_info_get_node_hostname_from_name_returns_hostname(input_dict: dict, display_name: str,
                                                               expected_hostname: str):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_hostname = env_info.get_node_hostname_from_name(display_name)

    # Assert
    assert actual_hostname == expected_hostname


@pytest.mark.parametrize(
    "input_dict,display_name,expected_url",
    [
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "target-node",
                        "url": "http://node89876-local-sha-master-0954606.hidora.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", "http://node89876-local-sha-master-0954606.hidora.com",
            id="list of 1 node with desired display name"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "irrelevant-node",
                        "url": "http://node89876-local-sha-master-0954606.hidora.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", None,
            id="list of 1 node without the desired display name"
        ),
        pytest.param(
            {
                "nodes": [
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.248",
                        "displayName": "irrelevant-node",
                        "url": "http://node89876-local-sha-master-0954606.hidora.com"
                    },
                    {
                        "nodeGroup": "cp",
                        "nodeType": "docker",
                        "intIP": "10.102.15.268",
                        "displayName": "target-node",
                        "url": "http://node89886-local-sha-master-0954656.hidora.com"
                    }
                ],
                "env": {
                    "envName": "local-sha-master-0954606",
                    "status": 1
                }
            }, "target-node", "http://node89886-local-sha-master-0954656.hidora.com",
            id="list of multiple nodes with the desired display name"
        ),
    ]
)
def test_env_info_get_node_url_from_name_returns_url(
        input_dict: dict, display_name: str, expected_url: str):
    # Arrange
    env_info = EnvInfo(input_dict)

    # Act
    actual_url = env_info.get_node_url_from_name(display_name)

    # Assert
    assert actual_url == expected_url
